@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include <qjsonobject.h>
 #include <qjsonvalue.h>
@@ -406,7 +407,7 @@ void QJsonArray::insert(qsizetype i, const QJsonValue &value)
     Inserts \a value before the position pointed to by \a before, and returns an iterator
     pointing to the newly inserted item.
 
-    \sa erase(), insert()
+    \sa erase(), insert(qsizetype, const QJsonValue &)
 */
 
 /*!
