@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qcursor.h"
 
@@ -150,7 +114,7 @@ QT_BEGIN_NAMESPACE
          \li Qt::DragLinkCursor      \li \c dnd-link or \c link
     \endtable
 
-    \sa QWidget, {fowler}{GUI Design Handbook: Cursors}
+    \sa QWidget
 */
 
 /*!
@@ -173,8 +137,7 @@ QT_BEGIN_NAMESPACE
 
 /*!
   \fn void QCursor::swap(QCursor &other)
-
-  Swaps this cursor with the \a other cursor.
+    \memberswap{cursor}
 
   \since 5.7
  */
@@ -326,7 +289,7 @@ QDataStream &operator<<(QDataStream &s, const QCursor &c)
         if (isPixmap)
             s << c.pixmap();
         else
-            s << c.bitmap(Qt::ReturnByValue) << c.mask(Qt::ReturnByValue);
+            s << c.bitmap() << c.mask();
         s << c.hotSpot();
     }
     return s;
@@ -475,8 +438,7 @@ QCursor::QCursor()
 QCursor::QCursor(Qt::CursorShape shape)
     : d(nullptr)
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     setShape(shape);
 }
 
@@ -534,8 +496,7 @@ bool operator==(const QCursor &lhs, const QCursor &rhs) noexcept
 */
 Qt::CursorShape QCursor::shape() const
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     return d->cshape;
 }
 
@@ -548,8 +509,7 @@ Qt::CursorShape QCursor::shape() const
 */
 void QCursor::setShape(Qt::CursorShape shape)
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     QCursorData *c = uint(shape) <= Qt::LastCursor ? qt_cursorTable[shape] : nullptr;
     if (!c)
         c = qt_cursorTable[0];
@@ -583,8 +543,7 @@ void QCursor::setShape(Qt::CursorShape shape)
 */
 QBitmap QCursor::bitmap() const
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     if (d->bm)
         return *(d->bm);
     return QBitmap();
@@ -610,8 +569,7 @@ QBitmap QCursor::bitmap() const
 */
 QBitmap QCursor::mask() const
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     if (d->bmm)
         return *(d->bmm);
     return QBitmap();
@@ -624,8 +582,7 @@ QBitmap QCursor::mask() const
 
 QPixmap QCursor::pixmap() const
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     return d->pixmap;
 }
 
@@ -636,8 +593,7 @@ QPixmap QCursor::pixmap() const
 
 QPoint QCursor::hotSpot() const
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     return QPoint(d->hx, d->hy);
 }
 
@@ -647,8 +603,7 @@ QPoint QCursor::hotSpot() const
 
 QCursor::QCursor(const QCursor &c)
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     d = c.d;
     d->ref.ref();
 }
@@ -671,8 +626,7 @@ QCursor::~QCursor()
 
 QCursor &QCursor::operator=(const QCursor &c)
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     if (c.d)
         c.d->ref.ref();
     if (d && !d->ref.deref())
@@ -743,8 +697,7 @@ void QCursorData::initialize()
 
 QCursorData *QCursorData::setBitmap(const QBitmap &bitmap, const QBitmap &mask, int hotX, int hotY, qreal devicePixelRatio)
 {
-    if (!QCursorData::initialized)
-        QCursorData::initialize();
+    QCursorData::initialize();
     if (bitmap.depth() != 1 || mask.depth() != 1 || bitmap.size() != mask.size()) {
         qWarning("QCursor: Cannot create bitmap cursor; invalid bitmap(s)");
         QCursorData *c = qt_cursorTable[0];
