@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qlocalserver.h"
 #include "qlocalserver_p.h"
@@ -197,7 +198,7 @@ qintptr QLocalServer::socketDescriptor() const
     return d->tcpServer.socketDescriptor();
 #elif defined(Q_OS_WIN)
     const auto handle = d->connectionEventNotifier->handle();
-    return handle != INVALID_HANDLE_VALUE ? qintptr(handle) : -1;
+    return handle ? qintptr(handle) : -1;
 #else
     return d->socketNotifier->socket();
 #endif
