@@ -17,7 +17,9 @@
 #include "qandroidplatformfontdatabase.h"
 #include "qandroidplatformforeignwindow.h"
 #include "qandroidplatformoffscreensurface.h"
+#if QT_CONFIG(egl)
 #include "qandroidplatformopenglcontext.h"
+#endif
 #include "qandroidplatformopenglwindow.h"
 #include "qandroidplatformscreen.h"
 #include "qandroidplatformservices.h"
@@ -29,7 +31,9 @@
 #include <QOpenGLContext>
 #include <QThread>
 #include <QtCore/QJniObject>
+#if QT_CONFIG(egl)
 #include <QtGui/private/qeglpbuffer_p.h>
+#endif
 #include <QtGui/private/qguiapplication_p.h>
 #include <QtGui/private/qoffscreensurface_p.h>
 #include <QtGui/private/qrhibackingstore_p.h>
@@ -138,6 +142,7 @@ void *QAndroidPlatformNativeInterface::nativeResourceForWindow(const QByteArray
 
 void *QAndroidPlatformNativeInterface::nativeResourceForContext(const QByteArray &resource, QOpenGLContext *context)
 {
+#if QT_CONFIG(egl)
     if (QEGLPlatformContext *platformContext = static_cast<QEGLPlatformContext *>(context->handle())) {
         if (resource == "eglcontext")
             return platformContext->eglContext();
@@ -146,6 +151,10 @@ void *QAndroidPlatformNativeInterface::nativeResourceForContext(const QByteArray
         else if (resource == "egldisplay")
             return platformContext->eglDisplay();
     }
+#else
+    Q_UNUSED(resource)
+    Q_UNUSED(context)
+#endif
     return nullptr;
 }
 
@@ -175,6 +184,7 @@ QAndroidPlatformIntegration::QAndroidPlatformIntegration(const QStringList &para
     Q_UNUSED(paramList);
     m_androidPlatformNativeInterface = new QAndroidPlatformNativeInterface();
 
+#if QT_CONFIG(egl)
     m_eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
     if (Q_UNLIKELY(m_eglDisplay == EGL_NO_DISPLAY))
         qFatal("Could not open egl display");
@@ -185,6 +195,7 @@ QAndroidPlatformIntegration::QAndroidPlatformIntegration(const QStringList &para
 
     if (Q_UNLIKELY(!eglBindAPI(EGL_OPENGL_ES_API)))
         qFatal("Could not bind GL_ES API");
+#endif
 
     using namespace QtJniTypes;
     m_primaryDisplayId = Display::getStaticField<jint>("DEFAULT_DISPLAY");
@@ -334,6 +345,7 @@ bool QAndroidPlatformIntegration::hasCapability(Capability cap) const
         // FIXME QTBUG-118849 - we do not implement grabWindow() anymore, calling it will return
         // a null QPixmap also for raster windows - for OpenGL windows this was always true
         case ScreenWindowGrabbing: return false;
+        case OffscreenSurface: return QtAndroidPrivate::activity().isValid();
         default:
             return QPlatformIntegration::hasCapability(cap);
     }
@@ -347,6 +359,7 @@ QPlatformBackingStore *QAndroidPlatformIntegration::createPlatformBackingStore(Q
     return new QRhiBackingStore(window);
 }
 
+#if QT_CONFIG(egl)
 QPlatformOpenGLContext *QAndroidPlatformIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
     if (!isValidAndroidContextForRendering())
@@ -389,6 +402,7 @@ QOffscreenSurface *QAndroidPlatformIntegration::createOffscreenSurface(ANativeWi
     surfacePrivate->platformOffscreenSurface = new QAndroidPlatformOffscreenSurface(nativeSurface, m_eglDisplay, surface);
     return surface;
 }
+#endif
 
 QPlatformWindow *QAndroidPlatformIntegration::createPlatformWindow(QWindow *window) const
 {
@@ -400,7 +414,11 @@ QPlatformWindow *QAndroidPlatformIntegration::createPlatformWindow(QWindow *wind
         return new QAndroidPlatformVulkanWindow(window);
 #endif
 
+#if QT_CONFIG(egl)
     return new QAndroidPlatformOpenGLWindow(window, m_eglDisplay);
+#endif
+
+    return nullptr;
 }
 
 QPlatformWindow *QAndroidPlatformIntegration::createForeignWindow(QWindow *window, WId nativeHandle) const
@@ -415,8 +433,10 @@ QAbstractEventDispatcher *QAndroidPlatformIntegration::createEventDispatcher() c
 
 QAndroidPlatformIntegration::~QAndroidPlatformIntegration()
 {
+#if QT_CONFIG(egl)
     if (m_eglDisplay != EGL_NO_DISPLAY)
         eglTerminate(m_eglDisplay);
+#endif
 
     delete m_androidPlatformNativeInterface;
     delete m_androidFDB;
