@@ -1,41 +1,7 @@
-/***************************************************************************
-**
-** Copyright (C) 2013 BlackBerry Limited. All rights reserved.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2013 BlackBerry Limited. All rights reserved.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+
+#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
 
 #include "qqnxglobal.h"
 
@@ -51,6 +17,7 @@
 #include "qqnxabstractvirtualkeyboard.h"
 #include "qqnxservices.h"
 
+#include "qqnxforeignwindow.h"
 #include "qqnxrasterwindow.h"
 #if !defined(QT_NO_OPENGL)
 #include "qqnxeglwindow.h"
@@ -65,11 +32,12 @@
 #if QT_CONFIG(qqnx_pps)
 #  include "qqnxbuttoneventnotifier.h"
 #  include "qqnxclipboard.h"
-#  if QT_CONFIG(qqnx_imf)
-#    include "qqnxinputcontext_imf.h"
-#  else
-#    include "qqnxinputcontext_noimf.h"
-#  endif
+#endif
+
+#if QT_CONFIG(qqnx_imf)
+#  include "qqnxinputcontext_imf.h"
+#else
+#  include "qqnxinputcontext_noimf.h"
 #endif
 
 #include <qpa/qplatforminputcontextfactory_p.h>
@@ -82,6 +50,7 @@
 #include <qpa/qwindowsysteminterface.h>
 
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/private/qrhibackingstore_p.h>
 
 #if !defined(QT_NO_OPENGL)
 #include "qqnxglcontext.h"
@@ -97,45 +66,47 @@
 #include <QtCore/QFile>
 #include <errno.h>
 
-#if defined(QQNXINTEGRATION_DEBUG)
-#define qIntegrationDebug qDebug
-#else
-#define qIntegrationDebug QT_NO_QDEBUG_MACRO
-#endif
-
 QT_BEGIN_NAMESPACE
 
+// Q_LOGGING_CATEGORY(lcQpaQnx, "qt.qpa.qnx");
+
+using namespace Qt::StringLiterals;
+
 QQnxIntegration *QQnxIntegration::ms_instance;
 
 static inline QQnxIntegration::Options parseOptions(const QStringList &paramList)
 {
     QQnxIntegration::Options options = QQnxIntegration::NoOptions;
-    if (!paramList.contains(QLatin1String("no-fullscreen"))) {
+    if (!paramList.contains("no-fullscreen"_L1)) {
         options |= QQnxIntegration::FullScreenApplication;
     }
 
-    if (paramList.contains(QLatin1String("flush-screen-context"))) {
+    if (paramList.contains("flush-screen-context"_L1)) {
         options |= QQnxIntegration::AlwaysFlushScreenContext;
     }
 
-    if (paramList.contains(QLatin1String("rootwindow"))) {
+    if (paramList.contains("rootwindow"_L1)) {
         options |= QQnxIntegration::RootWindow;
     }
 
-    if (!paramList.contains(QLatin1String("disable-EGL_KHR_surfaceless_context"))) {
+    if (!paramList.contains("disable-EGL_KHR_surfaceless_context"_L1)) {
         options |= QQnxIntegration::SurfacelessEGLContext;
     }
 
+    if (paramList.contains("desktop"_L1)) {
+        options |= QQnxIntegration::Desktop;
+    }
+
     return options;
 }
 
 static inline int getContextCapabilities(const QStringList &paramList)
 {
-    QString contextCapabilitiesPrefix = QStringLiteral("screen-context-capabilities=");
+    constexpr auto contextCapabilitiesPrefix = "screen-context-capabilities="_L1;
     int contextCapabilities = SCREEN_APPLICATION_CONTEXT;
     for (const QString &param : paramList) {
         if (param.startsWith(contextCapabilitiesPrefix)) {
-            QStringRef value = param.midRef(contextCapabilitiesPrefix.length());
+            auto value = QStringView{param}.mid(contextCapabilitiesPrefix.length());
             bool ok = false;
             contextCapabilities = value.toInt(&ok, 0);
             if (!ok)
@@ -147,16 +118,16 @@ static inline int getContextCapabilities(const QStringList &paramList)
 
 QQnxIntegration::QQnxIntegration(const QStringList &paramList)
     : QPlatformIntegration()
+    , m_screenContextId(256, 0)
     , m_screenEventThread(0)
     , m_navigatorEventHandler(new QQnxNavigatorEventHandler())
     , m_virtualKeyboard(0)
+    , m_inputContext(0)
 #if QT_CONFIG(qqnx_pps)
     , m_navigatorEventNotifier(0)
-    , m_inputContext(0)
     , m_buttonsNotifier(new QQnxButtonEventNotifier())
 #endif
     , m_qpaInputContext(0)
-    , m_services(0)
     , m_fontDatabase(new QGenericUnixFontDatabase())
     , m_eventDispatcher(createUnixEventDispatcher())
     , m_nativeInterface(new QQnxNativeInterface(this))
@@ -168,16 +139,24 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
 #if QT_CONFIG(draganddrop)
     , m_drag(new QSimpleDrag())
 #endif
+#if QT_CONFIG(opengl)
+    , m_eglDisplay(EGL_NO_DISPLAY)
+#endif
 {
     ms_instance = this;
     m_options = parseOptions(paramList);
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Open connection to QNX composition manager
     if (screen_create_context(&m_screenContext, getContextCapabilities(paramList))) {
         qFatal("%s - Screen: Failed to create screen context - Error: %s (%i)",
                Q_FUNC_INFO, strerror(errno), errno);
     }
+    screen_get_context_property_cv(m_screenContext,
+                                   SCREEN_PROPERTY_ID,
+                                   m_screenContextId.size(),
+                                   m_screenContextId.data());
+    m_screenContextId.resize(strlen(m_screenContextId.constData()));
 
 #if QT_CONFIG(qqnx_pps)
     // Create/start navigator event notifier
@@ -188,9 +167,8 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
     QMetaObject::invokeMethod(m_navigatorEventNotifier, "start", Qt::QueuedConnection);
 #endif
 
-#if !defined(QT_NO_OPENGL)
-    // Initialize global OpenGL resources
-    QQnxGLContext::initializeContext();
+#if QT_CONFIG(opengl)
+    createEglDisplay();
 #endif
 
     // Create/start event thread
@@ -215,10 +193,6 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
     m_navigator = new QQnxNavigatorPps();
 #endif
 
-    // Create services handling class
-    if (m_navigator)
-        m_services = new QQnxServices(m_navigator);
-
     createDisplays();
 
     if (m_virtualKeyboard) {
@@ -244,7 +218,7 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
 
 QQnxIntegration::~QQnxIntegration()
 {
-    qIntegrationDebug("platform plugin shutdown begin");
+    qCDebug(lcQpaQnx) << "Platform plugin shutdown begin";
     delete m_nativeInterface;
 
 #if QT_CONFIG(draganddrop)
@@ -277,9 +251,8 @@ QQnxIntegration::~QQnxIntegration()
     // Close connection to QNX composition manager
     screen_destroy_context(m_screenContext);
 
-#if !defined(QT_NO_OPENGL)
-    // Cleanup global OpenGL resources
-    QQnxGLContext::shutdownContext();
+#if QT_CONFIG(opengl)
+    destroyEglDisplay();
 #endif
 
 #if QT_CONFIG(qqnx_pps)
@@ -302,14 +275,15 @@ QQnxIntegration::~QQnxIntegration()
 
     ms_instance = nullptr;
 
-    qIntegrationDebug("platform plugin shutdown end");
+    qCDebug(lcQpaQnx) << "Platform plugin shutdown end";
 }
 
 bool QQnxIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     switch (cap) {
     case MultipleWindows:
+    case ForeignWindows:
     case ThreadedPixmaps:
         return true;
 #if !defined(QT_NO_OPENGL)
@@ -323,9 +297,21 @@ bool QQnxIntegration::hasCapability(QPlatformIntegration::Capability cap) const
     }
 }
 
+QPlatformWindow *QQnxIntegration::createForeignWindow(QWindow *window, WId nativeHandle) const
+{
+    screen_window_t screenWindow = reinterpret_cast<screen_window_t>(nativeHandle);
+    if (this->window(screenWindow)) {
+        qWarning() << "QWindow already created for foreign window"
+                   << screenWindow;
+        return nullptr;
+    }
+
+    return new QQnxForeignWindow(window, m_screenContext, screenWindow);
+}
+
 QPlatformWindow *QQnxIntegration::createPlatformWindow(QWindow *window) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QSurface::SurfaceType surfaceType = window->surfaceType();
     const bool needRootWindow = options() & RootWindow;
     switch (surfaceType) {
@@ -343,14 +329,25 @@ QPlatformWindow *QQnxIntegration::createPlatformWindow(QWindow *window) const
 
 QPlatformBackingStore *QQnxIntegration::createPlatformBackingStore(QWindow *window) const
 {
-    qIntegrationDebug();
-    return new QQnxRasterBackingStore(window);
+    QSurface::SurfaceType surfaceType = window->surfaceType();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO << surfaceType;
+    switch (surfaceType) {
+    case QSurface::RasterSurface:
+        return new QQnxRasterBackingStore(window);
+#if !defined(QT_NO_OPENGL)
+    // Return a QRhiBackingStore for non-raster surface windows
+    case QSurface::OpenGLSurface:
+        return new QRhiBackingStore(window);
+#endif
+    default:
+        return nullptr;
+    }
 }
 
 #if !defined(QT_NO_OPENGL)
 QPlatformOpenGLContext *QQnxIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Get color channel sizes from window format
     QSurfaceFormat format = context->format();
@@ -408,7 +405,7 @@ QPlatformOpenGLContext *QQnxIntegration::createPlatformOpenGLContext(QOpenGLCont
 
 QPlatformInputContext *QQnxIntegration::inputContext() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     if (m_qpaInputContext)
         return m_qpaInputContext;
     return m_inputContext;
@@ -416,7 +413,7 @@ QPlatformInputContext *QQnxIntegration::inputContext() const
 
 void QQnxIntegration::moveToScreen(QWindow *window, int screen)
 {
-    qIntegrationDebug() << "w =" << window << ", s =" << screen;
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO << "w =" << window << ", s =" << screen;
 
     // get platform window used by widget
     QQnxWindow *platformWindow = static_cast<QQnxWindow *>(window->handle());
@@ -430,7 +427,7 @@ void QQnxIntegration::moveToScreen(QWindow *window, int screen)
 
 QAbstractEventDispatcher *QQnxIntegration::createEventDispatcher() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // We transfer ownersip of the event-dispatcher to QtCoreApplication
     QAbstractEventDispatcher *eventDispatcher = m_eventDispatcher;
@@ -447,7 +444,7 @@ QPlatformNativeInterface *QQnxIntegration::nativeInterface() const
 #if !defined(QT_NO_CLIPBOARD)
 QPlatformClipboard *QQnxIntegration::clipboard() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
 #if QT_CONFIG(qqnx_pps)
     if (!m_clipboard)
@@ -466,7 +463,7 @@ QPlatformDrag *QQnxIntegration::drag() const
 
 QVariant QQnxIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     if ((hint == ShowIsFullScreen) && (m_options & FullScreenApplication))
         return true;
 
@@ -475,12 +472,16 @@ QVariant QQnxIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 
 QPlatformServices * QQnxIntegration::services() const
 {
+    // Create services handling class
+    if (m_navigator && !m_services)
+        m_services = new QQnxServices(m_navigator);
+
     return m_services;
 }
 
-QWindow *QQnxIntegration::window(screen_window_t qnxWindow)
+QWindow *QQnxIntegration::window(screen_window_t qnxWindow) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     return m_windowMapper.value(qnxWindow, 0);
@@ -488,7 +489,7 @@ QWindow *QQnxIntegration::window(screen_window_t qnxWindow)
 
 void QQnxIntegration::addWindow(screen_window_t qnxWindow, QWindow *window)
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     m_windowMapper.insert(qnxWindow, window);
@@ -496,7 +497,7 @@ void QQnxIntegration::addWindow(screen_window_t qnxWindow, QWindow *window)
 
 void QQnxIntegration::removeWindow(screen_window_t qnxWindow)
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     m_windowMapper.remove(qnxWindow);
@@ -547,7 +548,7 @@ static bool getRequestedDisplays(QJsonArray &requestedDisplays)
 
     // Read the requested display order
     const QJsonObject object = doc.object();
-    requestedDisplays = object.value(QLatin1String("displayOrder")).toArray();
+    requestedDisplays = object.value("displayOrder"_L1).toArray();
 
     return true;
 }
@@ -582,17 +583,16 @@ QList<screen_display_t *> QQnxIntegration::sortDisplays(screen_display_t *availa
 
     // Go through all the requested displays IDs
     QList<screen_display_t *> orderedDisplays;
-    for (const QJsonValue &value : qAsConst(requestedDisplays)) {
+    for (const QJsonValue &value : std::as_const(requestedDisplays)) {
         int requestedValue = value.toInt();
 
         // Move all displays with matching ID from the intermediate list
         // to the beginning of the ordered list
-        QMutableListIterator<screen_display_t *> iter(allDisplays);
-        while (iter.hasNext()) {
-            screen_display_t *display = iter.next();
+        for (auto it = allDisplays.begin(), end = allDisplays.end(); it != end; ++it) {
+            screen_display_t *display = *it;
             if (getIdOfDisplay(*display) == requestedValue) {
                 orderedDisplays.append(display);
-                iter.remove();
+                allDisplays.erase(it);
                 break;
             }
         }
@@ -606,7 +606,7 @@ QList<screen_display_t *> QQnxIntegration::sortDisplays(screen_display_t *availa
 
 void QQnxIntegration::createDisplays()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     // Query number of displays
     int displayCount = 0;
     int result = screen_get_context_property_iv(m_screenContext, SCREEN_PROPERTY_DISPLAY_COUNT,
@@ -636,11 +636,12 @@ void QQnxIntegration::createDisplays()
         Q_SCREEN_CHECKERROR(result, "Failed to query display attachment");
 
         if (!isAttached) {
-            qIntegrationDebug("Skipping non-attached display %d", i);
+            qCDebug(lcQpaQnx) << "Skipping non-attached display " << i;
             continue;
         }
 
-        qIntegrationDebug("Creating screen for display %d", i);
+        qCDebug(lcQpaQnx) << "Creating screen for display " << i;
+
         createDisplay(*orderedDisplays[i], /*isPrimary=*/false);
     } // of displays iteration
 }
@@ -674,7 +675,8 @@ void QQnxIntegration::removeDisplay(QQnxScreen *screen)
 
 void QQnxIntegration::destroyDisplays()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
+
     Q_FOREACH (QQnxScreen *screen, m_screens) {
         QWindowSystemInterface::handleScreenRemoved(screen);
     }
@@ -706,6 +708,11 @@ screen_context_t QQnxIntegration::screenContext()
     return m_screenContext;
 }
 
+QByteArray QQnxIntegration::screenContextId()
+{
+    return m_screenContextId;
+}
+
 QQnxNavigatorEventHandler *QQnxIntegration::navigatorEventHandler()
 {
     return m_navigatorEventHandler;
@@ -717,4 +724,28 @@ bool QQnxIntegration::supportsNavigatorEvents() const
     return m_navigator != 0;
 }
 
+#if QT_CONFIG(opengl)
+void QQnxIntegration::createEglDisplay()
+{
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
+
+    // Initialize connection to EGL
+    m_eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
+    if (Q_UNLIKELY(m_eglDisplay == EGL_NO_DISPLAY))
+        qFatal("QQnxiIntegration: failed to obtain EGL display: %x", eglGetError());
+
+    EGLBoolean eglResult = eglInitialize(m_eglDisplay, 0, 0);
+    if (Q_UNLIKELY(eglResult != EGL_TRUE))
+        qFatal("QQnxIntegration: failed to initialize EGL display, err=%d", eglGetError());
+}
+
+void QQnxIntegration::destroyEglDisplay()
+{
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
+
+    // Close connection to EGL
+    eglTerminate(m_eglDisplay);
+}
+#endif
+
 QT_END_NAMESPACE
