@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QDIALOG_P_H
 #define QDIALOG_P_H
@@ -51,7 +52,7 @@ public:
         {}
     ~QDialogPrivate();
 
-    virtual void setVisible(bool visible);
+    void setVisible(bool visible) override;
 
     QWindow *transientParentWindow() const;
     bool setNativeDialogVisible(bool visible);
