@@ -13,6 +13,9 @@
 #if defined(Q_OS_WIN)
 #  include <qt_windows.h>
 #endif
+#ifdef Q_OS_ANDROID
+#include <private/qjnihelpers_p.h>
+#endif
 
 #ifdef Q_OS_UNIX
 #include <unistd.h>
@@ -430,7 +433,7 @@ void tst_qstandardpaths::testAppConfigLocation()
 #endif
 }
 
-#ifndef Q_OS_WIN
+#if !defined(Q_OS_WIN) && !defined(Q_OS_WASM)
 // Find "sh" on Unix.
 // It may exist twice, in /bin/sh and /usr/bin/sh, in that case use the PATH order.
 static inline QFileInfo findSh()
@@ -484,6 +487,7 @@ void tst_qstandardpaths::testFindExecutable_data()
             << QString() << logo << logoPath;
     }
 #else
+# ifndef Q_OS_WASM
     const QFileInfo shFi = findSh();
     Q_ASSERT(shFi.exists());
     const QString shPath = shFi.absoluteFilePath();
@@ -493,6 +497,7 @@ void tst_qstandardpaths::testFindExecutable_data()
         << QString() << shPath << shPath;
     QTest::newRow("unix-sh-relativepath")
         << QString(shFi.absolutePath()) << QString::fromLatin1("./sh") << shPath;
+#endif /* !WASM */
 #endif
     QTest::newRow("idontexist")
         << QString() << QString::fromLatin1("idontexist") << QString();
@@ -524,16 +529,25 @@ void tst_qstandardpaths::testFindExecutable()
 
 void tst_qstandardpaths::testFindExecutableLinkToDirectory()
 {
+#ifdef Q_OS_WASM
+    QSKIP("No applicationdir on wasm");
+#else
+    const QString appPath = QCoreApplication::applicationDirPath();
+#ifdef Q_OS_ANDROID
+    if (QtAndroidPrivate::isUncompressedNativeLibs())
+        QSKIP("Can't create a link to applicationDir which points inside an APK on Android");
+#endif
     // link to directory
     const QString target = QDir::tempPath() + QDir::separator() + QLatin1String("link.lnk");
     QFile::remove(target);
-    QFile appFile(QCoreApplication::applicationDirPath());
+    QFile appFile(appPath);
     QVERIFY(appFile.link(target));
     QVERIFY(QStandardPaths::findExecutable(target).isEmpty());
     QFile::remove(target);
+#endif
 }
 
-using RuntimeDirSetup = QString (*)(QDir &);
+using RuntimeDirSetup = std::optional<QString> (*)(QDir &);
 Q_DECLARE_METATYPE(RuntimeDirSetup);
 
 void tst_qstandardpaths::testRuntimeDirectory()
@@ -593,18 +607,18 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         QSKIP("Running this test as root doesn't make sense");
 #  endif
 
-    addRow("environment:non-existing", [](QDir &d) {
+    addRow("environment:non-existing", [](QDir &d) -> std::optional<QString> {
         return updateRuntimeDir(d.filePath("runtime"));
     });
 
-    addRow("environment:existing", [](QDir &d) {
+    addRow("environment:existing", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("runtime");
         d.mkdir("runtime");
         QFile::setPermissions(p, QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner);
         return updateRuntimeDir(p);
     });
 
-    addRow("environment-to-existing-wrong-perm", [](QDir &d) {
+    addRow("environment-to-existing-wrong-perm", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("runtime");
         d.mkdir("runtime");
         QFile::setPermissions(p, QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner |
@@ -617,7 +631,7 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("environment:wrong-owner", [](QDir &) {
+    addRow("environment:wrong-owner", [](QDir &) -> std::optional<QString> {
         QT_STATBUF st;
         QT_STAT("/", &st);
 
@@ -632,10 +646,18 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("environment:file", [](QDir &d) {
+    // static so that it can be used in RuntimeDirSetup callable without capturing
+    static auto failedToOpen = [](const QFile &f) {
+        qCritical("QFile::Open: failed to open '%s': %s",
+                  qPrintable(f.fileName()), qPrintable(f.errorString()));
+        return std::nullopt;
+    };
+
+    addRow("environment:file", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("file");
         QFile f(p);
-        f.open(QIODevice::WriteOnly);
+        if (!f.open(QIODevice::WriteOnly))
+            return failedToOpen(f);
         f.setPermissions(QFile::ReadOwner | QFile::WriteOwner);
 
         updateRuntimeDir(p);
@@ -646,7 +668,7 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("environment:broken-symlink", [](QDir &d) {
+    addRow("environment:broken-symlink", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("link");
         QFile::link(d.filePath("this-goes-nowhere"), p);
         updateRuntimeDir(p);
@@ -657,7 +679,7 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("environment:symlink-to-dir", [](QDir &d) {
+    addRow("environment:symlink-to-dir", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("link");
         d.mkdir("dir");
         QFile::link(d.filePath("dir"), p);
@@ -670,12 +692,12 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("no-environment:non-existing", [](QDir &) {
+    addRow("no-environment:non-existing", [](QDir &) -> std::optional<QString> {
         clearRuntimeDir();
         return fallbackXdgRuntimeDir();
     });
 
-    addRow("no-environment:existing", [](QDir &d) {
+    addRow("no-environment:existing", [](QDir &d) -> std::optional<QString> {
         clearRuntimeDir();
         QString p = fallbackXdgRuntimeDir();
         d.mkdir(p);         // probably has wrong permissions
@@ -683,10 +705,11 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return p;
     });
 
-    addRow("no-environment:fallback-is-file", [](QDir &) {
+    addRow("no-environment:fallback-is-file", [](QDir &) -> std::optional<QString> {
         QString p = fallbackXdgRuntimeDir();
         QFile f(p);
-        f.open(QIODevice::WriteOnly);
+        if (!f.open(QIODevice::WriteOnly))
+            return failedToOpen(f);
         f.setPermissions(QFile::ReadOwner | QFile::WriteOwner);
 
         clearRuntimeDir();
@@ -697,10 +720,11 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
         return QString();
     });
 
-    addRow("environment-and-fallback-are-files", [](QDir &d) {
+    addRow("environment-and-fallback-are-files", [](QDir &d) -> std::optional<QString> {
         QString p = d.filePath("file1");
         QFile f(p);
-        f.open(QIODevice::WriteOnly);
+        if (!f.open(QIODevice::WriteOnly))
+            return failedToOpen(f);
         f.setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadGroup);
         updateRuntimeDir(p);
         QTest::ignoreMessage(QtWarningMsg,
@@ -710,7 +734,8 @@ void tst_qstandardpaths::testCustomRuntimeDirectory_data()
 
         f.close();
         f.setFileName(fallbackXdgRuntimeDir());
-        f.open(QIODevice::WriteOnly);
+        if (!f.open(QIODevice::WriteOnly))
+            return failedToOpen(f);
         f.setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadGroup);
         QTest::ignoreMessage(QtWarningMsg,
                              QString("QStandardPaths: runtime directory '%1' is not a directory, "
@@ -750,7 +775,9 @@ void tst_qstandardpaths::testCustomRuntimeDirectory()
     qputenv("TMPDIR", QFile::encodeName(tempDir.path()));
 
     QFETCH(RuntimeDirSetup, setup);
-    QString expected = setup(d);
+    std::optional<QString> opt = setup(d);
+    QVERIFY(opt);
+    QString expected = *opt;
 
     QString runtimeDir = QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation);
     QCOMPARE(runtimeDir, expected);
