@@ -1,31 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtCore/QUrl>
 #include <QTest>
@@ -41,6 +16,8 @@
 #define STRINGPREP_BIDI_BOTH_L_AND_RAL 4
 #define STRINGPREP_BIDI_LEADTRAIL_NOT_RAL 5
 
+using namespace Qt::StringLiterals;
+
 struct char16array {
     char16array() {}
     template <size_t N>
@@ -80,6 +57,9 @@ private Q_SLOTS:
     void encodingRecode();
     void encodingRecodeInvalidUtf8_data();
     void encodingRecodeInvalidUtf8();
+
+    void tldRestrictions_data();
+    void tldRestrictions();
 };
 #include "tst_qurlinternal.moc"
 
@@ -259,8 +239,8 @@ void tst_QUrlInternal::ace_testsuite_data()
     QTest::newRow("ascii-upper") << "FLUKE" << "fluke" << "fluke" << "fluke";
 
     // U+FB01 LATIN SMALL LIGATURE FI
-    QTest::newRow("asciifolded") << u"\uFB01le"_qs << "file" << "." << "file";
-    QTest::newRow("asciifolded-dotcom") << u"\uFB01le.example.com"_qs << "file.example.com" << "." << "file.example.com";
+    QTest::newRow("asciifolded") << u"\uFB01le"_s << "file" << "." << "file";
+    QTest::newRow("asciifolded-dotcom") << u"\uFB01le.example.com"_s << "file.example.com" << "." << "file.example.com";
     QTest::newRow("greek-mu") << QString::fromLatin1("\265V")
                               <<"xn--v-lmb"
                               << "."
@@ -744,10 +724,96 @@ void tst_QUrlInternal::encodingRecodeInvalidUtf8()
     output = QTest::currentDataTag();
     if (!qt_urlRecode(output, input, QUrl::FullyEncoded))
         output += input;
-    for (int i = int(strlen(QTest::currentDataTag())); i < output.length(); ++i) {
+    for (int i = int(strlen(QTest::currentDataTag())); i < output.size(); ++i) {
         QVERIFY2(output.at(i).unicode() < 0x80 || output.at(i) == QChar::ReplacementCharacter,
-                 qPrintable(QString("Character at i == %1 was U+%2").arg(i).arg(output.at(i).unicode(), 4, 16, QLatin1Char('0'))));
+                 qPrintable(QString("Character at i == %1 was U+%2").arg(i)
+                            .arg(ushort{output.at(i).unicode()}, 4, 16, u'0')));
     }
 }
 
+void tst_QUrlInternal::tldRestrictions_data()
+{
+    QTest::addColumn<QString>("tld");
+    QTest::addColumn<bool>("encode");
+
+    // current whitelist
+    QTest::newRow("ac")  << QString("ac")  << true;
+    QTest::newRow("ar")  << QString("ar")  << true;
+    QTest::newRow("asia")  << QString("asia")  << true;
+    QTest::newRow("at") << QString("at") << true;
+    QTest::newRow("biz")  << QString("biz")  << true;
+    QTest::newRow("br") << QString("br") << true;
+    QTest::newRow("cat")  << QString("cat")  << true;
+    QTest::newRow("ch")  << QString("ch")  << true;
+    QTest::newRow("cl")  << QString("cl")  << true;
+    QTest::newRow("cn") << QString("cn") << true;
+    QTest::newRow("com")  << QString("com")  << true;
+    QTest::newRow("de")  << QString("de")  << true;
+    QTest::newRow("dk") << QString("dk") << true;
+    QTest::newRow("es")  << QString("es")  << true;
+    QTest::newRow("fi") << QString("fi") << true;
+    QTest::newRow("gr")  << QString("gr")  << true;
+    QTest::newRow("hu") << QString("hu") << true;
+    QTest::newRow("il")  << QString("il")  << true;
+    QTest::newRow("info")  << QString("info")  << true;
+    QTest::newRow("io") << QString("io") << true;
+    QTest::newRow("is")  << QString("is")  << true;
+    QTest::newRow("ir")  << QString("ir")  << true;
+    QTest::newRow("jp") << QString("jp") << true;
+    QTest::newRow("kr") << QString("kr") << true;
+    QTest::newRow("li")  << QString("li")  << true;
+    QTest::newRow("lt") << QString("lt") << true;
+    QTest::newRow("lu")  << QString("lu")  << true;
+    QTest::newRow("lv")  << QString("lv")  << true;
+    QTest::newRow("museum") << QString("museum") << true;
+    QTest::newRow("name")  << QString("name")  << true;
+    QTest::newRow("net")  << QString("name")  << true;
+    QTest::newRow("no") << QString("no") << true;
+    QTest::newRow("nu")  << QString("nu")  << true;
+    QTest::newRow("nz")  << QString("nz")  << true;
+    QTest::newRow("org")  << QString("org")  << true;
+    QTest::newRow("pl")  << QString("pl")  << true;
+    QTest::newRow("pr")  << QString("pr")  << true;
+    QTest::newRow("se")  << QString("se")  << true;
+    QTest::newRow("sh") << QString("sh") << true;
+    QTest::newRow("tel")  << QString("tel")  << true;
+    QTest::newRow("th")  << QString("th")  << true;
+    QTest::newRow("tm")  << QString("tm")  << true;
+    QTest::newRow("tw") << QString("tw") << true;
+    QTest::newRow("ua")  << QString("ua")  << true;
+    QTest::newRow("vn") << QString("vn") << true;
+
+    // known blacklists:
+    QTest::newRow("foo") << QString("foo") << false;
+}
+
+void tst_QUrlInternal::tldRestrictions()
+{
+    QFETCH(QString, tld);
+
+    // www.brød.tld
+    QByteArray ascii = "www.xn--brd-1na." + tld.toLatin1();
+    QString unicode = QLatin1String("www.br\370d.") + tld;
+    QString encoded = QUrl::fromAce(ascii);
+    QTEST(!encoded.contains(".xn--"), "encode");
+    QTEST(encoded == unicode, "encode");
+
+    QUrl url = QUrl::fromEncoded("http://www.xn--brd-1na." + tld.toLatin1());
+    QTEST(!url.host().contains(".xn--"), "encode");
+    QTEST(url.host() == unicode, "encode");
+
+    url.setUrl(QLatin1String("http://www.xn--brd-1na.") + tld);
+    QTEST(!url.host().contains(".xn--"), "encode");
+    QTEST(url.host() == unicode, "encode");
+
+    url.setUrl(QLatin1String("http://www.br\370d.") + tld);
+    QTEST(!url.host().contains(".xn--"), "encode");
+    QTEST(url.host() == unicode, "encode");
+
+    url = QUrl::fromEncoded("http://www.br%C3%B8d." + tld.toLatin1());
+    QTEST(!url.host().contains(".xn--"), "encode");
+    QTEST(url.host() == unicode, "encode");
+}
+
+
 QTEST_APPLESS_MAIN(tst_QUrlInternal)
