@@ -1,39 +1,16 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
-#include <QtTest/QtTest>
+#include <QTest>
+#include <QTemporaryFile>
+#include <QSignalSpy>
+#include <QStandardPaths>
 
 #include <qcoreapplication.h>
 #include <qdebug.h>
 #include <qfiledialog.h>
 #include <qabstractitemdelegate.h>
-#include <qitemdelegate.h>
 #include <qlistview.h>
 #include <qcombobox.h>
 #include <qpushbutton.h>
@@ -48,6 +25,8 @@
 #include <qlayout.h>
 #include <qmenu.h>
 #include <qrandom.h>
+#include <qpointer.h>
+#include <qsettings.h>
 #include "../../../../../src/widgets/dialogs/qsidebar_p.h"
 #include "../../../../../src/gui/itemmodels/qfilesystemmodel_p.h"
 #include "../../../../../src/widgets/dialogs/qfiledialog_p.h"
@@ -57,9 +36,9 @@
 #include <qpa/qplatformdialoghelper.h>
 #include <qpa/qplatformintegration.h>
 
-#if defined(Q_OS_WIN)
-#include "../../../network-settings.h"
-#endif
+#include "../../../../shared/filesystem.h"
+
+#include <QtWidgets/private/qapplication_p.h>
 
 #if defined QT_BUILD_INTERNAL
 QT_BEGIN_NAMESPACE
@@ -127,9 +106,14 @@ private slots:
     void QTBUG4419_lineEditSelectAll();
     void QTBUG6558_showDirsOnly();
     void QTBUG4842_selectFilterWithHideNameFilterDetails();
+    void noCrashWhenParentIsDeleted();
     void dontShowCompleterOnRoot();
     void nameFilterParsing_data();
     void nameFilterParsing();
+#if QT_CONFIG(settings)
+    void settingsCompatibility_data();
+    void settingsCompatibility();
+#endif
 
 private:
     void cleanupSettingsFile();
@@ -278,7 +262,7 @@ void tst_QFileDialog2::unc()
 {
 #if defined(Q_OS_WIN)
     // Only test UNC on Windows./
-    QString dir("\\\\"  + QtNetworkSettings::winServerName() + "\\testsharewritable");
+    QString dir("\\\\"  + QTest::uncServerName() + "\\testsharewritable");
 #else
     QString dir(QDir::currentPath());
 #endif
@@ -330,7 +314,7 @@ static bool openContextMenu(QFileDialog &fd)
     MenuCloser closer(&fd);
     QObject::connect(&timer, &QTimer::timeout, &closer, &MenuCloser::close);
     timer.start();
-    QContextMenuEvent cme(QContextMenuEvent::Mouse, QPoint(10, 10));
+    QContextMenuEvent cme(QContextMenuEvent::Mouse, QPoint(10, 10), list->viewport()->mapToGlobal(QPoint(10, 10)));
     qApp->sendEvent(list->viewport(), &cme); // blocks until menu is closed again.
     return true;
 }
@@ -422,9 +406,9 @@ void tst_QFileDialog2::task180459_lastDirectory_data()
     QTest::addColumn<bool>("isEnabled");
     QTest::addColumn<QString>("result");
 
-    QTest::newRow("path+file") << QDir::homePath() + QDir::separator() + "foo"
+    QTest::newRow("path+file") << QDir::homePath() + QDir::separator() + "Vugiu1co"
             << QDir::homePath()  << true
-            << QDir::homePath() + QDir::separator() + "foo"  ;
+            << QDir::homePath() + QDir::separator() + "Vugiu1co"  ;
     QTest::newRow("no path") << ""
             << tempDir.path() << false << QString();
     QTest::newRow("file") << "foo"
@@ -470,6 +454,66 @@ void tst_QFileDialog2::task180459_lastDirectory()
     delete dlg;
 }
 
+#if QT_CONFIG(settings)
+void tst_QFileDialog2::settingsCompatibility_data()
+{
+    QTest::addColumn<QString>("qtVersion");
+    QTest::addColumn<QDataStream::Version>("dsVersion");
+    QTest::newRow("6.2.3") << "6.2.3" << QDataStream::Qt_6_0;
+    QTest::newRow("6.5") << "6.5" << QDataStream::Qt_5_0;
+    QTest::newRow("15.5.2") << "5.15.2" << QDataStream::Qt_5_15;
+    QTest::newRow("15.5.9") << "5.15.9" << QDataStream::Qt_5_15;
+}
+
+void tst_QFileDialog2::settingsCompatibility()
+{
+    static const QByteArray ba32 = QByteArrayLiteral("\x00\x00\x00\xFF\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\xF7\x00\x00\x00\x04\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00""d\xFF\xFF\xFF\xFF\x00\x00\x00\x81\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x01\t\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00>\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00""B\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00n\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03\xE8\x00\xFF\xFF\xFF\xFF\x00\x00\x00\x00");
+    static const QByteArray ba64 = QByteArrayLiteral("\x00\x00\x00\xFF\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\xF7\x00\x00\x00\x04\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00""d\xFF\xFF\xFF\xFF\x00\x00\x00\x81\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x01\t\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00>\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00""B\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00n\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x03\xE8\x00\xFF\xFF\xFF\xFF\x00\x00\x00\x00");
+    QByteArray impHeaderData;
+    QFETCH(QString, qtVersion);
+    QFETCH(QDataStream::Version, dsVersion);
+
+    // Create a header view, convert template to target format and store it in settings
+    {
+        QSettings settings(QSettings::UserScope, "QtProject");
+        settings.beginGroup("FileDialog");
+        settings.setValue("sidebarWidth", 93); // random value
+        settings.setValue("shortcuts", QStringList({settings.fileName(), "/tmp"}));
+        settings.setValue("qtVersion", qtVersion);
+        impHeaderData = dsVersion < QDataStream::Qt_6_0 ? ba32 : ba64;
+        settings.setValue("treeViewHeader", impHeaderData);
+        settings.endGroup();
+    }
+    // Create a file dialog, read settings write them back
+    {
+        QFileDialog fd;
+    }
+
+    // This test has been changed a bit since a settings readback and compare likely isn't very useful.
+    // It would either require that we could centrally (and/or decentrally) can set the saveState and
+    // restoreState versions (which currently isn't the case) or it will block
+    // any (also backward compatible) extension of saveState and restoreState (which isn't acceptable).
+    // So we check that we can indeed import the old data and that a transient saveResult will result
+    // in the same data.
+    // That we are decent forward compatible from newer to older versions is difficult to test.
+    QSettings settings(QSettings::UserScope, "QtProject");
+    settings.beginGroup("FileDialog");
+    const QByteArray savedState = settings.value("treeViewHeader").toByteArray();
+
+    // Check that we can restore data from an old version.
+    QHeaderView header1(Qt::Horizontal);
+    QVERIFY(header1.restoreState(impHeaderData));
+
+    // Check that we can restore the now saved data.
+    QHeaderView header2(Qt::Horizontal);
+    QVERIFY(header2.restoreState(savedState));
+
+    // Check that these states provide the same result
+    QByteArray h2data = header2.saveState();
+    QCOMPARE(header1.saveState(), h2data);
+    QCOMPARE(h2data, savedState);
+}
+#endif
 
 
 class FilterDirModel : public QSortFilterProxyModel
@@ -649,7 +693,7 @@ void tst_QFileDialog2::task226366_lowerCaseHardDriveWindows()
     //i clear my previous selection in the completer
     QTest::keyClick(edit->completer()->popup(), Qt::Key_Down);
     edit->clear();
-    QTest::keyClick(edit, (char)(Qt::Key_C | Qt::SHIFT));
+    QTest::keyClick(edit, Qt::Key_C, Qt::ShiftModifier);
     QTest::qWait(200);
     QTest::keyClick(edit->completer()->popup(), Qt::Key_Down);
     QCOMPARE(edit->text(), QString("C:/"));
@@ -662,13 +706,13 @@ void tst_QFileDialog2::completionOnLevelAfterRoot()
 #if defined(Q_OS_WIN)
     fd.setDirectory("C:/");
     QDir current = fd.directory();
-    QStringList entryList = current.entryList(QStringList(), QDir::Dirs);
+    const QStringList entryList = current.entryList(QStringList(), QDir::Dirs);
     // Find a suitable test dir under c:-root:
     // - At least 6 characters long
     // - Ascii, letters only
     // - No another dir with same start
     QString testDir;
-    foreach (const QString &entry, entryList) {
+    for (const QString &entry : entryList) {
         if (entry.size() > 5 && QString(entry.toLatin1()).compare(entry) == 0) {
             bool invalid = false;
             for (int i = 0; i < 5; i++) {
@@ -678,7 +722,7 @@ void tst_QFileDialog2::completionOnLevelAfterRoot()
                 }
             }
             if (!invalid) {
-                foreach (const QString &check, entryList) {
+                for (const QString &check : entryList) {
                     if (check.startsWith(entry.left(5), Qt::CaseInsensitive) && check != entry) {
                         invalid = true;
                         break;
@@ -693,6 +737,17 @@ void tst_QFileDialog2::completionOnLevelAfterRoot()
     }
     if (testDir.isEmpty())
         QSKIP("This test requires to have a unique directory of at least six ascii characters under c:/");
+#elif defined(Q_OS_ANDROID)
+    // Android 11 and above doesn't allow accessing root filesystem as before,
+    // so let's opt int for the app's home.
+    const auto homePaths = QStandardPaths::standardLocations(QStandardPaths::HomeLocation);
+    QVERIFY(!homePaths.isEmpty());
+    fd.setFilter(QDir::Hidden | QDir::AllDirs | QDir::Files | QDir::System);
+    fd.setDirectory(homePaths.first());
+    QDir(homePaths.first()).mkdir("etc");
+    auto cleanup = qScopeGuard([&]() {
+        QDir(homePaths.first()).rmdir("etc");
+    });
 #else
     fd.setFilter(QDir::Hidden | QDir::AllDirs | QDir::Files | QDir::System);
     fd.setDirectory("/");
@@ -774,8 +829,8 @@ void tst_QFileDialog2::task235069_hideOnEscape()
     child->setFocus();
     QTest::keyClick(child, Qt::Key_Escape);
     QCOMPARE(fd.isVisible(), false);
-    QCOMPARE(spyFinished.count(), 1); // QTBUG-7690
-    QCOMPARE(spyRejected.count(), 1); // reject(), don't hide()
+    QCOMPARE(spyFinished.size(), 1); // QTBUG-7690
+    QCOMPARE(spyRejected.size(), 1); // reject(), don't hide()
 }
 
 #ifdef QT_BUILD_INTERNAL
@@ -822,7 +877,7 @@ void tst_QFileDialog2::task203703_returnProperSeparator()
     QVERIFY(button);
     QTest::keyClick(button, Qt::Key_Return);
     QString result = fd.selectedFiles().first();
-    QVERIFY(result.at(result.count() - 1) != '/');
+    QVERIFY(result.at(result.size() - 1) != '/');
     QVERIFY(!result.contains('\\'));
     current.rmdir("aaaaaaaaaaaaaaaaaa");
 }
@@ -917,9 +972,9 @@ void tst_QFileDialog2::task239706_editableFilterCombo()
     d.show();
     QVERIFY(QTest::qWaitForWindowExposed(&d));
 
-    QList<QComboBox *> comboList = d.findChildren<QComboBox *>();
+    const QList<QComboBox *> comboList = d.findChildren<QComboBox *>();
     QComboBox *filterCombo = nullptr;
-    foreach (QComboBox *combo, comboList) {
+    for (QComboBox *combo : comboList) {
         if (combo->objectName() == QString("fileTypeCombo")) {
             filterCombo = combo;
             break;
@@ -997,10 +1052,10 @@ public :
     void removeSelection() {
         QList<QModelIndex> idxs = selectionModel()->selectedIndexes();
         QList<QPersistentModelIndex> indexes;
-        for (int i = 0; i < idxs.count(); i++)
+        for (int i = 0; i < idxs.size(); i++)
             indexes.append(idxs.at(i));
 
-        for (int i = 0; i < indexes.count(); ++i)
+        for (int i = 0; i < indexes.size(); ++i)
             if (!indexes.at(i).data(Qt::UserRole + 1).toUrl().path().isEmpty())
                 model()->removeRow(indexes.at(i).row());
     }
@@ -1080,7 +1135,6 @@ void tst_QFileDialog2::task254490_selectFileMultipleTimes()
     QTemporaryFile *t;
     t = new QTemporaryFile;
     QVERIFY2(t->open(), qPrintable(t->errorString()));
-    t->open();
     QFileDialog fd(0, "TestFileDialog");
 
     fd.setDirectory(tempPath);
@@ -1102,7 +1156,7 @@ void tst_QFileDialog2::task254490_selectFileMultipleTimes()
     QCOMPARE(lineEdit->text(),QLatin1String("new_file.txt"));
     QListView *list = fd.findChild<QListView*>("listView");
     QVERIFY(list);
-    QCOMPARE(list->selectionModel()->selectedRows(0).count(), 0);
+    QCOMPARE(list->selectionModel()->selectedRows(0).size(), 0);
 
     t->deleteLater();
 }
@@ -1118,7 +1172,7 @@ void tst_QFileDialog2::task257579_sideBarWithNonCleanUrls()
     QFileDialog fd;
     fd.setSidebarUrls(QList<QUrl>() << QUrl::fromLocalFile(url));
     QSidebar *sidebar = fd.findChild<QSidebar*>("sidebar");
-    QCOMPARE(sidebar->urls().count(), 1);
+    QCOMPARE(sidebar->urls().size(), 1);
     QVERIFY(sidebar->urls().first().toLocalFile() != url);
     QCOMPARE(sidebar->urls().first().toLocalFile(), QDir::cleanPath(url));
 
@@ -1223,7 +1277,7 @@ void tst_QFileDialog2::QTBUG6558_showDirsOnly()
 
     //Create a file
     QFile tempFile(dirPath + "/plop.txt");
-    tempFile.open(QIODevice::WriteOnly | QIODevice::Text);
+    QVERIFY(tempFile.open(QIODevice::WriteOnly | QIODevice::Text));
     QTextStream out(&tempFile);
     out << "The magic number is: " << 49 << "\n";
     tempFile.close();
@@ -1236,7 +1290,6 @@ void tst_QFileDialog2::QTBUG6558_showDirsOnly()
     fd.setOption(QFileDialog::ShowDirsOnly, true);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
@@ -1280,7 +1333,6 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
     fd.selectNameFilter(chosenFilterString);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
@@ -1296,7 +1348,6 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
     fd2.selectNameFilter(chosenFilterString);
     fd2.show();
 
-    QApplication::setActiveWindow(&fd2);
     QVERIFY(QTest::qWaitForWindowActive(&fd2));
     QCOMPARE(fd2.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd2));
@@ -1307,6 +1358,53 @@ void tst_QFileDialog2::QTBUG4842_selectFilterWithHideNameFilterDetails()
 
 }
 
+void tst_QFileDialog2::noCrashWhenParentIsDeleted()
+{
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url = QFileDialog::getOpenFileUrl(mainWindow.get(),
+                                                     QStringLiteral("getOpenFileUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url = QFileDialog::getSaveFileUrl(mainWindow.get(),
+                                                     QStringLiteral("getSaveFileUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QUrl url
+                = QFileDialog::getExistingDirectoryUrl(mainWindow.get(),
+                                                       QStringLiteral("getExistingDirectoryUrl"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!url.isValid());
+        QVERIFY(!mainWindow.get());
+    }
+
+    {
+        QPointer<QWidget> mainWindow = new QWidget();
+        QTimer::singleShot(1000, mainWindow, [mainWindow]
+                           { if (mainWindow.get()) mainWindow->deleteLater(); });
+        const QList<QUrl> url = QFileDialog::getOpenFileUrls(mainWindow.get(),
+                                                      QStringLiteral("getOpenFileUrls"));
+        QVERIFY(url.isEmpty());
+        QVERIFY(!mainWindow.get());
+    }
+}
+
 void tst_QFileDialog2::dontShowCompleterOnRoot()
 {
     if (!QGuiApplicationPrivate::platformIntegration()->hasCapability(QPlatformIntegration::WindowActivation))
@@ -1316,7 +1414,6 @@ void tst_QFileDialog2::dontShowCompleterOnRoot()
     fd.setAcceptMode(QFileDialog::AcceptSave);
     fd.show();
 
-    QApplication::setActiveWindow(&fd);
     QVERIFY(QTest::qWaitForWindowActive(&fd));
     QCOMPARE(fd.isVisible(), true);
     QCOMPARE(QApplication::activeWindow(), static_cast<QWidget*>(&fd));
