@@ -8,10 +8,12 @@
 #include <QPushButton>
 #include <QMainWindow>
 #include <QMenuBar>
+#include <QPlainTextEdit>
 #include <QToolBar>
 #include <QToolButton>
 #include <QStatusBar>
 #include <QListWidget>
+#include <QVBoxLayout>
 #include <QWidgetAction>
 #include <QScreen>
 #include <QSpinBox>
@@ -96,6 +98,8 @@ private slots:
     void QTBUG_89082_actionTipsHide();
     void QTBUG8122_widgetActionCrashOnClose();
     void widgetActionTriggerClosesMenu();
+    void widgetActionContextMenu();
+
     void transientParent();
 
     void QTBUG_10735_crashWithDialog();
@@ -122,6 +126,7 @@ private slots:
 #endif
 
     void invisibleActions();
+    void execReturnsWidgetAction();
 
 protected slots:
     void onActivated(QAction*);
@@ -1590,6 +1595,36 @@ void tst_QMenu::widgetActionTriggerClosesMenu()
     QCOMPARE(actionTriggered, &widgetAction);
 }
 
+void tst_QMenu::widgetActionContextMenu() // QTBUG-134757
+{
+    QPushButton openButton("open");
+    QMenu *menu = new QMenu(&openButton);
+    QVBoxLayout *layout = new QVBoxLayout;
+    QWidgetAction widgetAction(menu);
+    QWidget menuWidget(menu);
+    QPlainTextEdit edit;
+    openButton.setMenu(menu);
+    menuWidget.setLayout(layout);
+    widgetAction.setDefaultWidget(&menuWidget);
+    menu->addAction(&widgetAction);
+    layout->addWidget(&edit);
+
+    openButton.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&openButton));
+
+    // Click the QPushButton to open its menu
+    QTest::mouseClick(&openButton, Qt::LeftButton);
+    QVERIFY(QTest::qWaitForWindowExposed(&menuWidget));
+    QWindow *popupWindow = edit.window()->windowHandle();
+    QVERIFY(popupWindow);
+    QCOMPARE(QApplication::activePopupWidget(), menu);
+
+    // Right-click the QPlainTextEdit to open its context menu
+    QTest::mouseClick(popupWindow, Qt::RightButton);
+    QVERIFY(qobject_cast<QMenu *>(QApplication::activePopupWidget()));
+    QCOMPARE_NE(QApplication::activePopupWidget(), menu);
+}
+
 void tst_QMenu::transientParent()
 {
     QMainWindow window;
@@ -1610,31 +1645,64 @@ void tst_QMenu::transientParent()
     menuBar->addMenu(editMenu);
     window.setMenuBar(menuBar);
 
-    // On Mac, we need to create native key events to test menu
-    // action activation, so skip this part of the test.
-#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
+    QMenu *bookmarksMenu = new QMenu(&window);
+    bookmarksMenu->addAction("This is KDE!");
+
+    QMenu *contextMenu = new QMenu(&window);
+    QAction *bookmarksAction = contextMenu->addAction("&Bookmarks");
+    bookmarksAction->setMenu(bookmarksMenu);
+
     window.show();
     QVERIFY(QTest::qWaitForWindowActive(&window));
     QWindow *topLevel = window.windowHandle();
     QVERIFY(topLevel);
 
+    // Show the standalone bookmarks menu. It should be a child of the main window.
+    bookmarksMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), topLevel);
+    bookmarksMenu->close();
+
+    // Show the bookmarks menu attached to the context menu. Even though the bookmarks menu is
+    // a child of the main window, its transient parent will be the context menu.
+    contextMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(contextMenu));
+    QVERIFY(contextMenu->windowHandle());
+    QCOMPARE(contextMenu->windowHandle()->transientParent(), topLevel);
+
+    contextMenu->setActiveAction(bookmarksAction);
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), contextMenu->windowHandle());
+    contextMenu->close();
+
+    // Show the standalone bookmarks menu. Its transient parent will be the main window again.
+    bookmarksMenu->popup(window.geometry().center());
+    QTRY_VERIFY(QTest::qWaitForWindowExposed(bookmarksMenu));
+    QVERIFY(bookmarksMenu->windowHandle());
+    QCOMPARE(bookmarksMenu->windowHandle()->transientParent(), topLevel);
+    bookmarksMenu->close();
+
+    // On Mac, we need to create native key events to test menu
+    // action activation, so skip this part of the test.
+#if QT_CONFIG(shortcut) && !defined(Q_OS_DARWIN)
     window.setFocus();
     QVERIFY(QTest::qWaitForWindowFocused(&window));
     QVERIFY(window.hasFocus());
 
     QTest::keyPress(&window, Qt::Key_F, Qt::AltModifier);
     QTRY_VERIFY(QTest::qWaitForWindowExposed(fileMenu));
-    if (fileMenu->isWindow() && fileMenu->window() && fileMenu->window()->windowHandle())
-        QVERIFY(fileMenu->window()->windowHandle()->transientParent());
+    QVERIFY(fileMenu->windowHandle());
+    QCOMPARE(fileMenu->windowHandle()->transientParent(), topLevel);
     QTest::keyRelease(fileMenu, Qt::Key_F, Qt::AltModifier);
 
     QTest::keyPress(fileMenu, Qt::Key_E, Qt::AltModifier);
     QTRY_VERIFY(QTest::qWaitForWindowExposed(editMenu));
-    if (editMenu->isWindow() && editMenu->window() && editMenu->window()->windowHandle())
-        QVERIFY(editMenu->window()->windowHandle()->transientParent());
+    QVERIFY(editMenu->windowHandle());
+    QCOMPARE(editMenu->windowHandle()->transientParent(), topLevel);
     QTest::keyRelease(editMenu, Qt::Key_E, Qt::AltModifier);
 #endif // QT_CONFIG(shortcut) && !Q_OS_DARWIN
-
 }
 
 class MyMenu : public QMenu
@@ -2211,5 +2279,26 @@ void tst_QMenu::dontSelectDisabledActionByShortcut()
 }
 #endif
 
+void tst_QMenu::execReturnsWidgetAction()
+{
+    QWidget window;
+
+    QMenu menu(&window);
+    QWidgetAction *widgetAction = new QWidgetAction(&menu);
+    QPushButton *menuButton = new QPushButton("Button", &menu);
+    widgetAction->setDefaultWidget(menuButton);
+    QObject::connect(menuButton, &QPushButton::clicked, widgetAction, &QAction::trigger);
+
+    menu.addAction("First");
+    menu.addAction(widgetAction);
+    menu.addAction("Last");
+
+    window.show();
+    QVERIFY(QTest::qWaitForWindowActive(&window));
+
+    QTimer::singleShot(0, menuButton, &QPushButton::click);
+    QCOMPARE(menu.exec(window.geometry().center()), widgetAction);
+}
+
 QTEST_MAIN(tst_QMenu)
 #include "tst_qmenu.moc"
